package org.bitbucket.bradleysmithllc.etlunit.io.file;

import java.io.IOException;
import java.util.*;

class OrderedDataFileWriterImpl implements DataFileWriter
{
	private final RelationalDataFileWriterImpl writerImpl;
	private final DataFileSchema schema;
	private final FlatFile flatFile;
	private final List<String> names;

	public OrderedDataFileWriterImpl(DataFile.DataConverter converter, FlatFile flatFile) throws IOException
	{
		writerImpl = new RelationalDataFileWriterImpl(converter, flatFile);
		schema = flatFile.getDataFileSchema();
		names = schema.getColumnNames();
		this.flatFile = flatFile;
	}

	@Override
	public void close() throws IOException
	{
		// run a sort operation on the set, then pass on to the relational writer
		Collections.sort(setData, new Comparator<Map<String, Object>>()
		{
			@Override
			public int compare(Map<String, Object> o1, Map<String, Object> o2)
			{
				Map<String, Object> m1 = (Map<String, Object>) o1;
				Map<String, Object> m2 = (Map<String, Object>) o2;

				// get the two hash codes
				OrderKey keyOne = (OrderKey) m1.get(FlatFile.KEY_NULL);
				OrderKey keyTwo = (OrderKey) m2.get(FlatFile.KEY_NULL);

				return keyOne.compareTo(keyTwo);
			}
		});

		// pass all the rows to the writer
		for (Map<String, Object> row : setData)
		{
			writerImpl.addRow(row);
		}

		writerImpl.close();
	}

	List<Map<String, Object>> setData = new ArrayList<Map<String, Object>>();

	@Override
	public void addRow(Map<String, Object> rowData) throws IOException
	{
		Map<String, Object> colData = new HashMap<String, Object>();
		colData.putAll(rowData);

		OrderKey ok = FlatFile.addOrderKey(flatFile.getDataFileSchema(), colData, null);
		colData.put(FlatFile.KEY_NULL, ok);

		setData.add(colData);
	}
}